/*
 * Binary Marble Clock - 16 Jun 2018 - DR 
 * 
 * Binary clock has two rows of leds, top row has 4 leds, bottom has 6 leds, shows; 
 *  - time, random colours - top row is hours [0..12], bottom row is minutes [0..59]. Shown in binary 
 *  - date, magenta for month on top row, yellow for date on bottom row. Shown in binary 
 *  - day of week, green in bottom row, [Mon..Sun]. Led off indicates day, all on is Sun 
 *  - temperature, red on bottom row in °C [0..59]. Shown in binary
 *  - humidity, blue. Top row is lh digit, bottom row is rh digit. Shown as two bcd nibbles
 *  
 *  Cycles thru all displays on the leds at specified rate.
 *  All features are displayed on an OLED display
 *  
 *  Has "bump" feature to sync time to known source.  Pressing bump button moves time to nearest hour 
 *  Has "daylight saving adjust" , set slide switch to summer or winter and press adjust button. 
 *    In summer postion one hours is added, in Winter position one hour is subtracted.
 *    
 *  Time is set thru serial monitor as follows; 
 *   - set KBSetTime to true and download  
 *   - serial monitor will prompt and confirm time
 *   - set KBSetTime and DEBUG to false and re-download for normal operation
 *   - (if you don't do the redownload the next restart expects a timeset operation)
 *   
 *  Hardware used is;
 *   - Nano controller
 *   - single string of ten WS2812 leds
 *   - DS3231 real time clock module
 *   - DHT22 temperature and humidity sensor
 *   - OLED module 128 x 32
 *   - two momentary contact normally open pushbuttons and one spst slide switch
 *      
 */
  

#include <DS3232RTC.h>            //http://github.com/JChristensen/DS3232RTC
#include <TimeLib.h>              //http://playground.arduino.cc/Code/Time
#include <Adafruit_NeoPixel.h>    // WS2812 led strip driver library
#include <DHT.h>                  // DHT temperature and humidity sensor library
#include <Streaming.h>          //http://arduiniana.org/libraries/streaming/
#include "U8glib.h"             // OLED display

#define DEBUG false                // [true | false]  turns on serial debugging - reduces sketch size a fair bit
#if (DEBUG)
  bool allow_show = true;
#endif

#define KBSetTime false          // {true | false} enables setting the time to the RTC module via keyboard
                                //   uses serial window and DEBUG must be true

const int M1 = 0;            // the first led in the string   
const int M2 = 1;
const int M4 = 2;
const int M8 = 3;
const int M16 = 4;
const int M32 = 5;

const int H1 = 6;
const int H2 = 7;
const int H4 = 8;
const int H8 = 9;            // the last led in the string   

const int BumpINTpin = 2;     // used as ISR for Bump to hr routine
                              // Int_0, from pin 2, is the highest priority execpt reset

const int DstINTpin = 3;      // used as ISR for Daylight saving adjust routines
                              // Int_1, from pin 3, is lower priority than Int_0
                   
const int ds_adjust = 4;      // switch input for daylight saying. 1 = in ds time, 0 = not ds (is a pullup)
const int DHTpin = 7;         // comms pin for the temp / humidity module, use a 10k pullup too
const int Ledpin = 8;         // comms pin for the led string, use 470 ohm series resistor
const int NumLeds = 10;       // physically arranged H8, H4, H2, H1, <some seperation> M32, M16, M8, M4, M2, M1

const char DegSymb = 176;  // the ° symbol 

volatile boolean IntFlag = false;           // interupt capture flag
volatile boolean DstBumpIntFlag = false;    // interupt capture flag for retard clock in autumn

#define Red      leds.Color(0xff, 0x00, 0x00)     // colour for temperature display
#define Green    leds.Color(0x00, 0xff, 0x00)     // colour for day of week display
#define Blue     leds.Color(0x00, 0x00, 0xff)     // colour for humidity display
#define Magenta  leds.Color(0xff, 0x00, 0xff)     // colour for month display
#define Yellow   leds.Color(0xff, 0xff, 0x00)     // colour for day display
#define Purple   leds.Color(0xb7, 0x00, 0xff)     // colour for fault display
#define Off      leds.Color(0x00, 0x00, 0x00)     // colour for clearing display

Adafruit_NeoPixel leds = Adafruit_NeoPixel (NumLeds, Ledpin, NEO_GRB + NEO_KHZ800);
DHT dht(DHTpin, DHT22);     // the temp & humidity sensor, (pin and sensor type)
U8GLIB_SSD1306_128X32 u8g(U8G_I2C_OPT_NONE);  // SSD1306 128 x32 I2C / TWI Oled module, SDA is A4, SCL is A5 

unsigned long elapsed, frame;
unsigned long prev = 0;

boolean  show_time_run = false;
boolean  show_dow_run = false;
boolean  show_mnthdte_run = false;
boolean  show_temp_run = false;
boolean  show_hum_run = false;

const unsigned long show_time_delay = 10000;    //these set the time that the relevant display is shown on the leds
const unsigned long show_dow_delay = 2000;
const unsigned long show_mnthdte_delay = 3000;
const unsigned long show_temp_delay = 4000;
const unsigned long show_hum_delay = 4000;

//Oled strings               nb. strings need a '\0' null terminator char
char temp_str[4];            // 0 .. 100  
char hum_str[4];             // 0 .. 100
char time_str[6];            // hh:mm
char mins_str[3];            // mm  [0 ..59]
char hrs_str[3];             // hh  [0 .. 23]  - maybe clipped to [0 .. 11]
char mnth_str[3];            // mm  [1 ..12]
char day_str[3];             // dd  [1 .. 31] 
char yr_str[5];              // yyyy
//char date_str[11];           // dd/mm/yyyy
char date_str[9];           // dd/mm/yy

time_t snapshot;

void loop() {

  if (IntFlag) ZeroSec();                //check for an interupt
  IntFlag = false;                       //   and clear it

  if (DstBumpIntFlag) DS_bump ();   //check for an interupt
  DstBumpIntFlag = false;                //   and clear it

   snapshot = now();      //get the system time, this to ensure consistency 

  if ((timeStatus() != timeSet)) {      // could be timeNotSet or timeNeedsSync
    // let user know a problem exists
    set_led_purple ();
    return;                             // nothing else happens
  }

  u8g.firstPage();            // manage the Oled update
  do {
    draw();
  } while( u8g.nextPage() );

  elapsed = millis () - prev;    // keeps track of time to update the led display
  
  if (not show_time_run) {
    show_time ();
    frame += show_time_delay;
    show_time_run = true;
    //Serial << frame << endl;
    }
  else if (elapsed > frame && not show_dow_run) {
    show_day_of_week ();
    frame += show_dow_delay;
    show_dow_run = true;
    }
  else if (elapsed > frame && not show_mnthdte_run) {
    show_month_date ();
    frame += show_mnthdte_delay;
    show_mnthdte_run = true;
    }
  else if (elapsed > frame && not show_temp_run) {
    show_temp ();
    frame += show_temp_delay;
    show_temp_run = true;
    }
  else if (elapsed > frame && not show_hum_run) {
    show_humidity ();
    frame += show_hum_delay;
    show_hum_run = true;
    }
  else if (elapsed > frame) {
    reset_display ();
  }
  

  #if (DEBUG)
    if ((second() == 0) && allow_show) {
      ShowSysTimestatus ();
      allow_show = false;
    }
    if (second() != 0) allow_show = true;
  #endif
}

void reset_display () {
  frame = 0;
  show_time_run = false;
  show_dow_run = false;
  show_mnthdte_run = false;
  show_temp_run = false;
  show_hum_run = false;
  prev = millis();
}


void Leds_off () {
  for (int i = 0; i < NumLeds; i++) {
    leds.setPixelColor (i, Off);
  }
}

int myst (int seed) {               //used to generate a random colour var
  randomSeed(seed);     // shake up the random sequence
  return random (256);
}

void set_led_purple () {

  Leds_off ();
  
  leds.setPixelColor (M32, Purple);
  leds.setPixelColor (M16, Purple);
  leds.setPixelColor (M8, Purple);
  leds.setPixelColor (M4, Purple);
  leds.setPixelColor (M2, Purple);
  leds.setPixelColor (M1, Purple);

  leds.setPixelColor (H8, Purple);
  leds.setPixelColor (H4, Purple);
  leds.setPixelColor (H2, Purple);
  leds.setPixelColor (H1, Purple);

  leds.show();
}

